// NeoPixel Ring simple sketch (c) 2013 Shae Erisson
// released under the GPLv3 license to match the rest of the AdaFruit NeoPixel library

#include <Wire.h>         // this #include still required because the RTClib depends on it
#include "RTClib.h"
#include <Rotary.h>

#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
  #include <avr/power.h>
#endif

enum MODE {
  M_OFF,
  CLOCK,
  ADJUSTING_ZEN,
  ZEN,
  ZEN_ENDED
};

MODE mode = CLOCK;
Rotary encoder = Rotary(2, 3);

#define INPUT_BUTTON_PIN  7
#define PIXELS_PIN        8

#define NUMPIXELS         17

#define FIRST_HOUR_LED 0
#define FIRST_MINUTE_LED 12
#define SECONDS_LED 16



uint8_t hours = 12;
uint8_t minutes = 55;
uint8_t seconds = 0;

RTC_DS1307 rtc;

// When we setup the NeoPixel library, we tell it how many pixels, and which pin to use to send signals.
// Note that for older NeoPixel strips you might need to change the third parameter--see the strandtest
// example for more information on possible values.
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(NUMPIXELS, PIXELS_PIN, NEO_GRB + NEO_KHZ800);

int delayval = 50; // delay for half a second

void setup() {
  // This is for Trinket 5V 16MHz, you can remove these three lines if you are not using a Trinket
#if defined (__AVR_ATtiny85__)
  if (F_CPU == 16000000) clock_prescale_set(clock_div_1);
#endif
  // End of trinket special code

  //rtc.begin(DateTime(F(__DATE__), F(__TIME__)));

  Serial.begin(9600);

  if(!rtc.begin()){
    Serial.println("No clock");
    while(1);
  }

  /*if(!rtc.isrunning()){
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
  }*/

  pinMode(INPUT_BUTTON_PIN, INPUT_PULLUP);
  
  pixels.begin(); // This initializes the NeoPixel library.
  pixels.setBrightness(100);
}

int timerMinutes = 0;
int timerHours = 0;
float timerSeconds = 0;

int pixelBrightness = 128;

void loop() {
  int delay_ms = 1000;

  uint32_t externalColor; 
  uint32_t internalColor;
  uint32_t centerColor;
  
  if(hours >= 7 && hours < 21)
  {
    externalColor = pixels.Color(21, 101, 192);
    internalColor = pixels.Color(79, 195, 247);
    centerColor = pixels.Color( 56, 142, 60);
  }
  else if(hours >= 21 || hours <= 7)
  {
    externalColor = pixels.Color(183,28,28);
    internalColor = pixels.Color(216,67,21);
    centerColor = pixels.Color(245,124,0);
  }

  unsigned char result = encoder.process();
  /*if(Serial.available())
  {
    char r = Serial.read();
    if(r == 's') switchMode();
    else if(r == 'a') result = DIR_CCW;
    else if(r == 'd') result = DIR_CW; 
  }*/

  if(digitalRead(INPUT_BUTTON_PIN) == 0)
  {
    switchMode();
  }
  
  if(mode == M_OFF){
    allPixels(pixels.Color(0, 0, 0));
    return;
  }
  else if(mode == CLOCK){
    if (result) {
      if(result == DIR_CW){
        increaseBrightness();
      } else{
        decreaseBrightness();
      }
    }
    
    DateTime now = rtc.now();
  
    hours = now.hour();
    minutes = now.minute();
    seconds = now.second();
    
    drawTime(externalColor, internalColor, centerColor);
      
    delay_ms = 100;
  } 
  else if(mode == ADJUSTING_ZEN)
  {
    if (result) {
      if(result == DIR_CW){
        timerMinutes += 15;
        if(timerMinutes >= 60)
        {
          timerMinutes = 0;
          timerHours++;
        }
        if(timerHours > 12)
        {
          timerHours = 0;
        }
      } else{
        timerMinutes -= 15;
        if(timerMinutes < 0)
        {
          timerMinutes = 45;
          timerHours--;
        }
        if(timerHours < 0)
          timerHours = 0;
      }
    }

    hours = timerHours;
    minutes = timerMinutes;
    drawTime(externalColor, internalColor, centerColor);
    delay_ms = 10;
  }
  else if(mode == ZEN)
  {
    if (result) {
      if(result == DIR_CW){
        increaseBrightness();
      } else{
        decreaseBrightness();
      }
    }

    timerSeconds += 0.1;
    if(timerSeconds >= 60)
    {
      timerSeconds = 0;
      timerMinutes--;
      if(timerMinutes <= 0){
        timerMinutes = 60;
        timerHours--;
        if(timerHours <= 0)
        {
          timerHours = 0;
          mode = ZEN_ENDED;
        }
      }
    }

    hours = timerHours;
    minutes = timerMinutes;
    seconds = timerSeconds;

    drawTime(externalColor, internalColor, centerColor);

    delay_ms = 100;
  }
  else if(mode == ZEN_ENDED)
  {
    allPixels(externalColor);
    delay(100);
    allPixels(internalColor);
    delay_ms = 100;
  }

  pixels.setBrightness(pixelBrightness);
  pixels.show();
  
  delay(delay_ms);
}

void allPixels(uint32_t color)
{
  for(int i = 0; i < NUMPIXELS; i++)
  {
    pixels.setPixelColor(i, color);
  }
  pixels.show();
}

void switchMode()
{
  switch(mode){
    case M_OFF:
      mode = CLOCK;
    break;
    case CLOCK:
      mode = ADJUSTING_ZEN;
      timerMinutes = 0;
      timerHours = 0;
      pixels.clear();
      pixels.show();
    break;
    case ADJUSTING_ZEN:
      mode = ZEN;
      if(timerMinutes == 0 && timerHours == 0) mode = M_OFF;
    break;
    case ZEN:
      mode = M_OFF;
    break;
    case ZEN_ENDED:
      mode = M_OFF;
      break;
  }

  Serial.println(mode);
}

void increaseBrightness()
{
  pixelBrightness += 16;
  if(pixelBrightness > 255) pixelBrightness = 255;
}

void decreaseBrightness()
{
  pixelBrightness -= 16;
  if(pixelBrightness < 0) pixelBrightness = 0;
}

void drawTime(uint32_t externalColor, uint32_t internalColor, uint32_t centerColor)
{
  pixels.clear();
  
  pixels.setPixelColor(FIRST_HOUR_LED + 11, externalColor);
  
  for(int i=FIRST_HOUR_LED; i < FIRST_HOUR_LED + (hours % 12); i++){   
    pixels.setPixelColor(i, externalColor);
  }

  if(minutes > 7)
    pixels.setPixelColor(FIRST_MINUTE_LED, internalColor);
  if(minutes > 22)
    pixels.setPixelColor(FIRST_MINUTE_LED + 1, internalColor);
  if(minutes > 37)
    pixels.setPixelColor(FIRST_MINUTE_LED + 2, internalColor);
   if(minutes > 52)
    pixels.setPixelColor(FIRST_MINUTE_LED + 3, internalColor); 

  if(seconds % 2 == 0)
    pixels.setPixelColor(SECONDS_LED, centerColor);
}
